"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateReactQuery = generateReactQuery;
const utils_1 = require("../utils");
function generateQueryOptions(operation, spec) {
    const { operationId, parameters, requestBody, method } = operation;
    const hasData = (parameters && parameters.length > 0) || operation.requestBody;
    // Helper to get required fields from a schema
    const getRequiredFields = (schema, context) => {
        if ("$ref" in schema) {
            const refType = schema.$ref.split("/").pop();
            const refSchema = context.schemas[refType];
            return refSchema?.required?.map((p) => `'${p}'`) || [];
        }
        return schema.required?.map((p) => `'${p}'`) || [];
    };
    // Get required parameter names from both parameters and request body
    const requiredParams = [
        ...(parameters?.filter((p) => p.required).map((p) => `'${p.name}'`) || []),
        ...(requestBody && "content" in requestBody && requestBody.content?.["application/json"]?.schema
            ? getRequiredFields(requestBody.content["application/json"].schema, {
                schemas: spec.components?.schemas || {},
            })
            : []),
        ...(requestBody && "content" in requestBody && requestBody.content?.["multipart/form-data"]?.schema
            ? getRequiredFields(requestBody.content["multipart/form-data"].schema, {
                schemas: spec.components?.schemas || {},
            })
            : []),
    ];
    const namedQueryOptions = (0, utils_1.camelCase)(`get${operationId}QueryOptions`);
    const namedQuery = (0, utils_1.camelCase)(`${operationId}`);
    return `
export const ${namedQueryOptions} = ( 
  ${hasData ? `props: Partial<Parameters<typeof apiClient.${namedQuery}>[0]>` : `props?: Partial<Parameters<typeof apiClient.${namedQuery}>[0]>`}
) => {
  ${hasData ? "const { axiosConfig, ...params } = props || {};" : "const { axiosConfig } = props || {};"}
  const enabled = ${hasData ? `hasDefinedProps(params, ${requiredParams.join(", ")})` : "true"};
  return queryOptions({
    queryKey: ['${(0, utils_1.camelCase)(operationId)}', ${hasData ? "params" : ""}],
    queryFn: enabled ? () => apiClient.${namedQuery}(${hasData ? "{...params, axiosConfig}" : "{axiosConfig}"}) : skipToken,
  });
};`;
}
function generateReactQuery(spec) {
    const operations = [];
    // Collect operations (same as in clientGenerator)
    Object.entries(spec.paths || {}).forEach(([path, pathItem]) => {
        if (!pathItem)
            return;
        ["get", "post", "put", "delete", "patch"].forEach((method) => {
            const operation = pathItem[method];
            if (!operation)
                return;
            operations.push({
                method: method,
                path,
                operationId: (0, utils_1.sanitizeTypeName)(`${operation.operationId || `${path.replace(/\W+/g, "_")}`}`),
                summary: operation.summary,
                description: operation.description,
                parameters: [
                    ...(pathItem.parameters || []),
                    ...(operation.parameters || []),
                ],
                requestBody: operation.requestBody,
                responses: operation.responses,
            });
        });
    });
    return `import { queryOptions, skipToken } from '@tanstack/react-query';
	import * as apiClient from './${(0, utils_1.specTitle)(spec)}.client';
	// TEMPORARY: allows for backward compatibility imports
	export * from './${(0, utils_1.specTitle)(spec)}.client';

const hasDefinedProps = <T extends { [P in K]?: any }, K extends PropertyKey>(
  obj: T,
  ...keys: K[]
): obj is T & { [P in K]-?: Exclude<T[P], undefined> } => {
  return keys.every((k) => obj[k] !== undefined);
};

${operations.map((op) => generateQueryOptions(op, spec)).join("\n\n")}
`;
}
